#
# File: make-project.mk
# Creator: George Ferguson
# Created: Wed Jun  2 11:27:41 2010
# Time-stamp: <Tue Jun 15 13:37:23 EDT 2010 ferguson>
#
# Makefile targets for building a project consisting of several packages.
#
# Copyright 2010 George Ferguson, ferguson@cs.rochester.edu.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Assumes the following are set:
#  SUBDIRS      List of package subdirectories to build (each with Package.mk)
#  JARFILE      Name of final jar file
#  PROJECT_MAIN Main class for jar
#

include config.mk

define include_subdir
SUBDIR := $(1)
include $(1)/Package.mk
SRC_CLASSES := $(SRC_CLASSES) $$(addprefix $$(SUBDIR)/,$$(SRCS:.java=.class))
ALL_CLASSES := $(ALL_CLASSES) $$(addprefix $$(SUBDIR)/,$$(CLASSES))
PACKAGE_FILES := $(PACKAGE_FILES) $$(addprefix $$(SUBDIR)/,$$(CLASSES) $$(EXTRA))
endef

$(foreach subdir,$(SUBDIRS),$(eval $(call include_subdir,$(subdir))))

all:: $(JARFILE)

MANIFEST = MANIFEST.MF
$(MANIFEST): Makefile
	echo "Main-Class: $(PROJECT_MAIN)" >$(MANIFEST)
	echo "SplashScreen-Image: $(PROJECT_SPLASH)" >>$(MANIFEST)

$(JARFILE): $(SRC_CLASSES) $(MANIFEST)
	$(JAR) cvfm $@ $(MANIFEST) $(subst $$,\$$,$(PACKAGE_FILES))

.SUFFIXES: .java .class
.java.class:
	$(JAVAC) $(JFLAGS) $<

clean::
	rm -f $(JARFILE) $(MANIFEST) $(subst $$,\$$,$(ALL_CLASSES))

run::
	$(JAVA) -jar $(JARFILE) $(JFLAGS) $(ARGV)

# File is loaded
PROJECT_MK = 1
